@@ -682,8 +682,9 @@ up_device_supply_refresh_battery (UpDeviceSupply *supply,
 	state = up_device_supply_get_state (native);
 
 	/* this is the new value in uW */
-	energy_rate = fabs (g_udev_device_get_sysfs_attr_as_double_uncached (native, "power_now") / 1000000.0);
-	if (energy_rate < 0.01) {
+	if (g_udev_device_has_sysfs_attr (native, "power_now")) {
+		energy_rate = fabs (g_udev_device_get_sysfs_attr_as_double_uncached (native, "power_now") / 1000000.0);
+	} else {
 		gdouble charge_full;
 
 		/* convert charge to energy */
@@ -723,8 +724,8 @@ up_device_supply_refresh_battery (UpDeviceSupply *supply,
 	if (energy_rate == 0xffff)
 		energy_rate = 0;
 
-	/* sanity check to less than 100W */
-	if (energy_rate > 100)
+	/* Ensure less than 300W, above the 240W possible with USB Power Delivery */
+	if (energy_rate > 300)
 		energy_rate = 0;
 
 	/* the hardware reporting failed -- try to calculate this */
@@ -1062,10 +1063,10 @@ up_device_supply_guess_type (GUdevDevice *native,
 
 		sibling = up_device_supply_get_sibling_with_subsystem (native, "input");
 		if (sibling) {
-			if (g_udev_device_get_property_as_boolean (sibling, "ID_INPUT_MOUSE")) {
-				type = UP_DEVICE_KIND_MOUSE;
-			} else if (g_udev_device_get_property_as_boolean (sibling, "ID_INPUT_TOUCHPAD")) {
+			if (g_udev_device_get_property_as_boolean (sibling, "ID_INPUT_TOUCHPAD")) {
 				type = UP_DEVICE_KIND_TOUCHPAD;
+			} else if (g_udev_device_get_property_as_boolean (sibling, "ID_INPUT_MOUSE")) {
+				type = UP_DEVICE_KIND_MOUSE;
 			} else if (g_udev_device_get_property_as_boolean (sibling, "ID_INPUT_JOYSTICK")) {
 				type = UP_DEVICE_KIND_GAMING_INPUT;
 			} else {
