@@ -46,6 +46,7 @@ struct UpDaemonPrivate
 	GHashTable		*poll_timeouts;
 	gboolean                 poll_paused;
 	GHashTable		*idle_signals;
+	int			 critical_action_lock_fd;
 
 	/* Properties */
 	UpDeviceLevel		 warning_level;
@@ -588,6 +589,12 @@ up_daemon_set_on_battery (UpDaemon *daemon, gboolean on_battery)
 static gboolean
 take_action_timeout_cb (UpDaemon *daemon)
 {
+	/* Release the inhibitor lock first, otherwise our action may be canceled */
+	if (daemon->priv->critical_action_lock_fd >= 0) {
+		close (daemon->priv->critical_action_lock_fd);
+		daemon->priv->critical_action_lock_fd = -1;
+	}
+
 	up_backend_take_action (daemon->priv->backend);
 
 	g_debug ("Backend was notified to take action. The timeout will be removed.");
@@ -618,6 +625,7 @@ up_daemon_set_warning_level (UpDaemon *daemon, UpDeviceLevel warning_level)
 	if (daemon->priv->warning_level == UP_DEVICE_LEVEL_ACTION) {
 		if (daemon->priv->action_timeout_id == 0) {
 			g_debug ("About to take action in %d seconds", UP_DAEMON_ACTION_DELAY);
+			daemon->priv->critical_action_lock_fd = up_backend_inhibitor_lock_take (daemon->priv->backend, "Execute critical action", "block");
 			daemon->priv->action_timeout_id = g_timeout_add_seconds (UP_DAEMON_ACTION_DELAY,
 										 (GSourceFunc) take_action_timeout_cb,
 										 daemon);
@@ -704,6 +712,40 @@ up_daemon_update_warning_level (UpDaemon *daemon)
 		up_daemon_set_warning_level (daemon, warning_level);
 }
 
+const gchar *
+up_daemon_get_charge_icon (UpDaemon     *daemon,
+			   gdouble       percentage,
+			   UpDeviceLevel battery_level,
+			   gboolean      charging)
+{
+	if (battery_level == UP_DEVICE_LEVEL_NONE && daemon != NULL) {
+		if (percentage <= daemon->priv->low_percentage)
+			return charging ? "battery-caution-charging-symbolic" : "battery-caution-symbolic";
+		else if (percentage < 30)
+			return charging ? "battery-low-charging-symbolic" : "battery-low-symbolic";
+		else if (percentage < 60)
+			return charging ? "battery-good-charging-symbolic" : "battery-good-symbolic";
+		return charging ? "battery-full-charging-symbolic" : "battery-full-symbolic";
+	} else {
+		switch (battery_level) {
+		case UP_DEVICE_LEVEL_UNKNOWN:
+			/* The lack of symmetry is on purpose */
+			return charging ? "battery-good-charging-symbolic" : "battery-caution-symbolic";
+		case UP_DEVICE_LEVEL_LOW:
+		case UP_DEVICE_LEVEL_CRITICAL:
+			return charging ? "battery-caution-charging-symbolic" : "battery-caution-symbolic";
+		case UP_DEVICE_LEVEL_NORMAL:
+			return charging ? "battery-low-charging-symbolic" : "battery-low-symbolic";
+		case UP_DEVICE_LEVEL_HIGH:
+			return charging ? "battery-good-charging-symbolic" : "battery-good-symbolic";
+		case UP_DEVICE_LEVEL_FULL:
+			return charging ? "battery-full-charging-symbolic" : "battery-full-symbolic";
+		default:
+			g_assert_not_reached ();
+		}
+	}
+}
+
 /**
  * up_daemon_device_changed_cb:
  **/
@@ -1039,8 +1081,8 @@ static void
 load_percentage_policy (UpDaemon    *daemon,
 			gboolean     load_default)
 {
-	LOAD_OR_DEFAULT (daemon->priv->low_percentage, "PercentageLow", 10);
-	LOAD_OR_DEFAULT (daemon->priv->critical_percentage, "PercentageCritical", 3);
+	LOAD_OR_DEFAULT (daemon->priv->low_percentage, "PercentageLow", 20);
+	LOAD_OR_DEFAULT (daemon->priv->critical_percentage, "PercentageCritical", 5);
 	LOAD_OR_DEFAULT (daemon->priv->action_percentage, "PercentageAction", 2);
 }
 
@@ -1154,6 +1196,11 @@ up_daemon_finalize (GObject *object)
 	if (priv->action_timeout_id != 0)
 		g_source_remove (priv->action_timeout_id);
 
+	if (priv->critical_action_lock_fd >= 0) {
+		close (priv->critical_action_lock_fd);
+		priv->critical_action_lock_fd = -1;
+	}
+
 	g_clear_pointer (&priv->poll_timeouts, g_hash_table_destroy);
 	g_clear_pointer (&priv->idle_signals, g_hash_table_destroy);
 
